package propellerbug;

/**
 *
 * @author matthew stevenson
 */
public class Main {

    public static void main(String[] args) {
        PropellerServoController psc = new PropellerServoController("/dev/ttyUSB0");
        try{
            psc.connect();
            System.out.println("PSCU Version: " + psc.getVersion());

            //Testing Low 85 for all channels, Baud Rate 2400
            for(int id=0; id<16; id++){
                for(int high=0; high<5; high++){
                    if(!testLow85(psc, id, high)){
                        System.out.println("Test Failed.  Expected to read 4 bytes for low 85, 3 bytes for all others");
                        System.out.println("Press any key to continue.");
                        System.in.read();
                    }
                }
            }
            //Testing Low 85 for all channels, Baud Rate 38400
            psc.setBaudRate(38400);
            for(int id=0; id<16; id++){
                for(int high=0; high<5; high++){
                    if(!testLow85(psc, id, high)){
                        System.out.println("Test Failed.  Expected to read 4 bytes for low 85, 3 bytes for all others");
                        System.out.println("Press any key to continue.");
                        System.in.read();
                    }
                }
            }
            //Testing Low 33 for all channels, Baud Rate 2400
            for(int id=0; id<16; id++){
                for(int high=0; high<5; high++){
                    if(!testLow33(psc, id, high)){
                        System.out.println("Test Failed.  Expected to ignore positions after moving to low 33");
                        System.out.println("Press any key to continue.");
                        System.in.read();
                    }
                }
            }
            //Testing Low 33 for all channels, Baud Rate 38400
            for(int id=0; id<16; id++){
                for(int high=0; high<5; high++){
                    psc.setBaudRate(38400);
                    if(!testLow33(psc, id, high)){
                        System.out.println("Test Failed.  Expected to ignore positions after moving to low 33");
                        System.out.println("Press any key to continue.");
                        System.in.read();
                    }
                }
            }
            psc.disconnect();
        }catch(Throwable t){
            t.printStackTrace();
        }
        System.exit(0);
    }

    public static boolean testLow85(PropellerServoController psc, int id, int high){
        System.out.println();
        System.out.println("=====================================================================");
        System.out.println("Testing Servo " + id + " with high " + high + ", expecting 4 bytes read for low 85.");
        System.out.println("=====================================================================");
        byte[] bytes;
        boolean passed = true;
        try{
            psc.moveServo((byte)id, (byte)0, (byte)84, (byte)high);
            Utils.sleep(50);
            bytes = psc.getPosition(id);
            if(bytes.length != 3){
                passed = false;
            }
            psc.moveServo((byte)id, (byte)0, (byte)85, (byte)high);
            Utils.sleep(50);
            bytes = psc.getPosition(id);
            if(bytes.length != 4){
                passed = false;
            }
            psc.moveServo((byte)id, (byte)0, (byte)86, (byte)high);
            Utils.sleep(50);
            bytes = psc.getPosition(id);
            if(bytes.length != 3){
                passed = false;
            }
            System.out.println("=====================================================================");
        }catch(Throwable t){
            t.printStackTrace();
        }
        return passed;
    }

    public static boolean testLow33(PropellerServoController psc, int id, int high){
        System.out.println();
        System.out.println("=====================================================================");
        System.out.println("Testing Servo " + id + " with high " + high + ", expecting servo disabled for low 33.");
        System.out.println("=====================================================================");
        byte[] bytes;
        boolean passed = true;
        try{
            psc.moveServo((byte)id, (byte)0, (byte)32, (byte)high);
            Utils.sleep(50);
            bytes = psc.getPosition(id);
            if(bytes[1] != high && bytes[2] != 32){
                passed = false;
            }
            psc.moveServo((byte)id, (byte)0, (byte)33, (byte)high);
            Utils.sleep(50);
            bytes = psc.getPosition(id);
            if(bytes[1] != high && bytes[2] != 33){
                passed = false;
            }
            psc.moveServo((byte)id, (byte)0, (byte)34, (byte)high);
            Utils.sleep(50);
            bytes = psc.getPosition(id);
            if(bytes[1] != high && bytes[2] != 33){
                passed = false;
            }
            psc.moveServo((byte)id, (byte)0, (byte)100, (byte)high);
            Utils.sleep(50);
            bytes = psc.getPosition(id);
            if(bytes[1] != high && bytes[2] != 33){
                passed = false;
            }
            psc.moveServo((byte)id, (byte)0, (byte)100, (byte)(high+1));
            Utils.sleep(50);
            bytes = psc.getPosition(id);
            if(bytes[1] != high && bytes[2] != 33){
                passed = false;
            }
            psc.moveServo((byte)id, (byte)0, (byte)10, (byte)high);
            Utils.sleep(50);
            bytes = psc.getPosition(id);
            if(bytes[1] != high && bytes[2] != 33){
                passed = false;
            }
            System.out.println("Reseting controller connection");
            System.out.println();
            Utils.sleep(50);
            psc.disconnect();
            Utils.sleep(50);
            psc.connect();
            System.out.println("PSCU Version: " + psc.getVersion());
            System.out.println("=====================================================================");
        }catch(Throwable t){
            t.printStackTrace();
        }
        return passed;
    }

}
